﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Zeiterfassung
{
    abstract class Person
    {
        string vorname;
        string nachname;
        string anrede;
        DateTime geburtsdatum;
        public Person(string vorname, string nachname, string anrede, DateTime geburtsdatum)
        {
            this.vorname = vorname;
            this.nachname = nachname;
            this.anrede = anrede;
            this.geburtsdatum = geburtsdatum;
        }

        public virtual bool DarfBüroAlleineBetreten()
        {
            return false;
        }

        public virtual int GibPersonalnummer()
        {
            return -1;
        }

        public virtual void Kommt()
        { }

        public virtual void Geht()
        { }

        public string Nachname
        { get { return nachname; } }
    }

    class Mitarbeiter : Person
    {
        List<KommtGehtZeit> kommtGehtZeiten = new List<KommtGehtZeit>();
        static int nächsteZuVergebendePersonalnummer = 0;
        int personalnummer;
        public Mitarbeiter(string vorname, string nachname, string anrede, DateTime geburtsdatum)
        : base(vorname, nachname, anrede, geburtsdatum)
        {
            personalnummer = nächsteZuVergebendePersonalnummer;
            nächsteZuVergebendePersonalnummer++;
        }
        public override bool DarfBüroAlleineBetreten()
        {
            return true;
        }
        public override int GibPersonalnummer()
        {
            return personalnummer;
        }
        public override string ToString()
        {
            return Nachname;
        }
        public override void Kommt()
        {
            if(kommtGehtZeiten.Count != 0
                && kommtGehtZeiten.Last().IstAnwesend)
            {
                throw new ApplicationException("Kommt/Geht-Fehler!");
            }
            kommtGehtZeiten.Add(new KommtGehtZeit());
        }
        public override void Geht()
        {
            if (kommtGehtZeiten.Count == 0)
            {
                throw new ApplicationException("Kommt/Geht-Fehler!");
            }
            //klassisch:
            //KommtGehtZeit letzte = kommtGehtZeiten[kommtGehtZeiten.Count-1];
            //letzte.Geht();
            kommtGehtZeiten.Last().Geht();
        }
        public TimeSpan BerechneArbeitszeit()
        {
            TimeSpan summe = new TimeSpan();
            // klassisch:
            //for (int i = 0; i < kommtGehtZeiten.Count; i++)
            //{
            //    summe += kommtGehtZeiten[i].Dauer;
            //}
            foreach (KommtGehtZeit vorgang in kommtGehtZeiten)
            {
                summe += vorgang.Dauer;
            }

            //Wenn Zahl der Stunden als double gefragt:
            // return kommtGehtZeiten.Sum(x => x.Dauer.TotalHours);

            return summe;
        }
    }

    class Gast : Person
    {
        Mitarbeiter gastgeber;
        public Gast(string vorname, string nachname, string anrede, DateTime geburtsdatum, Mitarbeiter gastgeber)
        : base(vorname, nachname, anrede, geburtsdatum)
        {
            if(gastgeber == null)
            {
                throw new ApplicationException("Gastgeber ist null.");
            }
            this.gastgeber = gastgeber;
        }

        KommtGehtZeit kommtGehtZeit;

        public override void Kommt()
        {
            if(kommtGehtZeit != null)
            {
                throw new ApplicationException("Instanz von Gast zweimal benutzt!");
            }
            kommtGehtZeit = new KommtGehtZeit();
        }

        public override void Geht()
        {
            if(kommtGehtZeit == null)
            {
                throw new ApplicationException("Kommen/Gehen-Fehler!");
            }
            kommtGehtZeit.Geht();
        }
    }

    class KommtGehtZeit
    {
        DateTime kommt;
        DateTime geht;
        bool istAnwesend = true;
        public KommtGehtZeit()
        {
            kommt = DateTime.Now;
        }

        public bool IstAnwesend
        { get { return istAnwesend; } }

        public void Geht()
        {
            if(!istAnwesend)
            {
                throw new ApplicationException("Kommt/Geht-Fehler!");
            }
            geht = DateTime.Now;
            istAnwesend = false;
        }

        public TimeSpan Dauer
        {
            get
            {
                if(istAnwesend)
                {
                    return DateTime.Now - kommt;
                }
                else
                {
                    return geht - kommt;
                }
            }
        }
    }

}
